/**
 * @file GParaboloidIndividual2D.hpp
 */

/*
 * Copyright (C) Gemfony scientific UG (haftungsbeschraenkt)
 *
 * See the AUTHORS file in the top-level directory for a list of authors.
 *
 * Contact: contact [at] gemfony (dot) eu
 *
 * This file is part of the Geneva library collection.
 *
 * Geneva was developed with kind support from Karlsruhe Institute of
 * Technology (KIT) and Steinbuch Centre for Computing (SCC). Further
 * information about KIT and SCC can be found at http://www.kit.edu/english
 * and http://scc.kit.edu .
 *
 * Geneva is free software: you can redistribute and/or modify it under
 * the terms of version 3 of the GNU Affero General Public License
 * as published by the Free Software Foundation.
 *
 * Geneva is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with the Geneva library. If not, see <http://www.gnu.org/licenses/>.
 *
 * For further information on Gemfony scientific and Geneva, visit
 * http://www.gemfony.eu .
 */

// Global checks, defines and includes needed for all of Geneva
#include "common/GGlobalDefines.hpp"

// Standard header files go here
#include <iostream>

// Boost header files go here

#ifndef GPARABOLOIDINDIVIDUAL2D_HPP_
#define GPARABOLOIDINDIVIDUAL2D_HPP_

// Geneva header files go here
#include <geneva/GParameterSet.hpp>
#include <geneva/GConstrainedDoubleObject.hpp>

namespace Gem {
namespace Geneva {

/******************************************************************/
/**
 * This individual searches for the minimum of a 2-dimensional parabola.
 * It is part of an introductory example, used in the Geneva manual.
 */
class GParaboloidIndividual2D :public GParameterSet
{
	 /** @brief Make the class accessible to Boost.Serialization */
	 friend class boost::serialization::access;

	 /**************************************************************/
	 /**
	  * This function triggers serialization of this class and its
	  * base classes.
	  */
	 template<typename Archive>
	 void serialize(Archive & ar, const unsigned int) {
		 using boost::serialization::make_nvp;
		 // Serialize the base class
		 ar & BOOST_SERIALIZATION_BASE_OBJECT_NVP(GParameterSet);
		 // Add other variables here like this:
		 // ar & BOOST_SERIALIZATION_NVP(sampleVariable);
	 }
	 /**************************************************************/
public:
	 /** @brief The default constructor */
	 GParaboloidIndividual2D();
	 /** @brief A standard copy constructor */
	 GParaboloidIndividual2D(const GParaboloidIndividual2D&);
	 /** @brief The standard destructor */
	 virtual ~GParaboloidIndividual2D();

protected:
	 /** @brief Loads the data of another GParaboloidIndividual2D */
	 virtual void load_(const GObject*) final;

	 /** @brief The actual fitness calculation takes place here. */
	 virtual double fitnessCalculation() final;

private:
	 /** @brief Creates a deep clone of this object */
	 virtual GObject* clone_() const final;

	 const double PAR_MIN_;
	 const double PAR_MAX_;
};

/******************************************************************/

} /* namespace Geneva */
} /* namespace Gem */

BOOST_CLASS_EXPORT_KEY(Gem::Geneva::GParaboloidIndividual2D)

#endif /* GPARABOLOIDINDIVIDUAL2D_HPP_ */
